﻿using System;
using System.Collections;
using System.Text;

namespace WindowsApplication11
{
	/// <summary>
	/// 把枚举值按照指定的文本显示
	/// <remarks>
	/// 一般通过枚举值的ToString()可以得到变量的文本，
	/// 但是有时候需要的到与之对应的更充分的文本，
	/// 这个类帮助达到此目的
	/// Date: 2006-3-25 
	/// Author: dearzp@hotmail.com
	/// </remarks>
	/// </summary>
	/// <example>
	/// [EnumDescription(MyEnum.One, "数字一")]
	/// [EnumDescription(MyEnum.Two, "数字二")]
	/// [EnumDescription(MyEnum.Three, "数字三")]
	/// enum MyEnum
	/// {
	/// 	One = 1, Two, Three
	/// }
	/// EnumDescription.GetText(typeof(MyEnum), (int)MyEnum.Eight);
	/// EnumDescription.GetTexts(typeof(MyEnum)); 
	/// </example>
	[AttributeUsage(AttributeTargets.Enum, AllowMultiple = true)]
	public class EnumDescription : Attribute
	{
		private int enumIntValue;
		private string enumDisplayText;
		private int enumRank;

		/// <summary>
		/// 描述枚举值
		/// </summary>
		/// <param name="enumValue">枚举值</param>
		/// <param name="enumDisplayText">描述内容</param>
		/// <param name="enumRank">排列顺序</param>
		public EnumDescription( object enumValue, string enumDisplayText, int enumRank )
		{
			try
			{ this.enumIntValue = (int)enumValue; }
			catch ( InvalidCastException )
			{ throw new ArgumentException("enumValue只能是原枚举定义或整型"); }

			this.enumDisplayText = enumDisplayText;
			this.enumRank = enumRank;
		}

		/// <summary>
		/// 描述枚举值，默认排序为5
		/// </summary>
		/// <param name="enumValue">枚举值</param>
		/// <param name="enumDisplayText">描述内容</param>
		public EnumDescription( object enumValue, string enumDisplayText )
			: this(enumValue, enumDisplayText, 5) { }


		public int EnumValue
		{
			get { return this.enumIntValue; }
		}

		public string EnumDisplayText
		{
			get { return this.enumDisplayText; }
		}

		public int EnumRank
		{
			get { return enumRank; }
		}

		#region  =========================================对枚举描述属性的解释相关函数

		/// <summary>
		/// 排序类型
		/// </summary>
		public enum SortType
		{
			/// <summary>
			///按枚举值排序
			/// </summary>
			Value,
			/// <summary>
			/// 按描述值排序
			/// </summary>
			DisplayText,
			/// <summary>
			/// 按排序熵
			/// </summary>
			Rank
		}

		private static System.Collections.Hashtable cachedEnum = new Hashtable();

		/// <summary>
		/// 获得指定枚举类型中，指定值的描述文本
		/// </summary>
		/// <exception cref="ArgumentException"></exception>
		/// <param name="enumType">枚举类型</param>
		/// <param name="enumValue">枚举类型值</param>
		/// <returns>返回定义的文本</returns>
		public static string GetText( Type enumType, int enumValue )
		{
			EnumDescription[] descriptions = GetTexts(enumType, SortType.Rank);
			foreach ( EnumDescription var in descriptions )
			{
				if ( var.EnumValue == enumValue ) return var.EnumDisplayText;
			}
			throw new ArgumentException("值[" + enumValue.ToString() + "]未在枚举类型[" + enumType.Name + "]中定义");
		}


		/// <summary>
		/// 得到枚举类型定义的所有文本
		/// </summary>
		/// <exception cref="NotSupportedException"></exception>
		/// <param name="enumType">枚举类型</param>
		/// <param name="sortType">指定排序类型</param>
		/// <returns>所有定义的文本</returns>
		public static EnumDescription[] GetTexts( Type enumType, SortType sortType )
		{
			EnumDescription[] descriptions = null;
			//缓存处理
			if ( cachedEnum.Contains(enumType.FullName) == false )
				cachedEnum.Add(enumType.FullName, enumType.GetCustomAttributes(typeof(EnumDescription), false));
			descriptions = (EnumDescription[])cachedEnum[enumType.FullName];
			if ( descriptions.Length <= 0 ) throw new NotSupportedException("枚举类型[" + enumType.Name + "]未定义属性EnumValueDescription");

			//按指定的属性冒泡排序
			for ( int m = 0; m < descriptions.Length; m++ )
			{
				for ( int n = m; n < descriptions.Length; n++ )
				{
					EnumDescription temp;
					bool swap = false;

					switch ( sortType )
					{
						case SortType.Value:
							if ( descriptions[m].EnumValue > descriptions[n].EnumValue ) swap = true;
							break;
						case SortType.DisplayText:
							if ( string.Compare(descriptions[m].EnumDisplayText, descriptions[n].EnumDisplayText) > 0 ) swap = true;
							break;
						case SortType.Rank:
							if ( descriptions[m].EnumRank > descriptions[n].EnumRank ) swap = true;
							break;
					}

					if ( swap )
					{
						temp = descriptions[m];
						descriptions[m] = descriptions[n];
						descriptions[n] = temp;
					}
				}
			}

			return descriptions;
		}

		#endregion
	}
}
